require "prefabutil"

local Paintable = Class(function(self, inst)
    self.inst = inst
	self.parent = nil
    
	self.fxdata = {}
    self.fxchildren = {}
    self.visible = false
end)

--- Add an effect to be spawned when burning
-- @param prefab The prefab to spawn as the effect
-- @color parent The parent thing

function Paintable:AddThing(prefab)
    table.insert(self.fxdata, {prefab = prefab})
end

function Paintable:SetParent(inst)
	self.parent = inst
end

function Paintable:SyncParentChild()

end

function Paintable:OnRemoveEntity()
	self:KillFX()
end

function Paintable:Paint(colors)
	--print("Painting...")
	
	if #self.fxdata > 0 then
		self:KillFX() --Stop and remove previous things
		
		for k, v in pairs(self.fxdata) do
			local fx = SpawnPrefab(v.prefab)
			table.insert(self.fxchildren, fx)
			
			self.inst:AddChild(fx)
			fx.Transform:SetPosition(0, 0, 0) --Might need to change these?
			
			--fx.AnimState:PlayAnimation("closed")
			--fx.AnimState:PushAnimation("closed")
			fx.AnimState:SetMultColour(0,0,0,1)
			fx.AnimState:SetRayTestOnBB(true)
			fx:SetStateGraph("SGflag")
			
			if colors then
				fx.AnimState:SetAddColour(	colors.r/255,
											colors.g/255,
											colors.b/255, 
													0)
			else
				fx.AnimState:SetAddColour(	1, 
											2,
											3, 
											0)
			end
		end
	else
		print("ERROR: Nothing to paint!")
	end
end

function Paintable:PlayAnimation(animation, doloop)
	--print("sync-ing...")
	for k, v in pairs(self.fxchildren) do
		v.AnimState:PlayAnimation(animation, doloop)
	end
end

function Paintable:GoToStates(state)
	--print("Going to states...")
	for k, v in pairs(self.fxchildren) do
		v.sg:GoToState(state)
	end
end

--Remove all paintable things
function Paintable:KillFX()
	for k, v in pairs(self.fxchildren) do
		--v:ListenForEvent("animover", function(inst) inst:Remove() end)
        v:Remove()
		self.fxchildren[k] = nil
	end
end

--[[
function Paintable:OnRemoveFromEntity()
    self:Extinguish()
    if self.task then
        self.task:Cancel()
        self.task = nil
    end
end
--]]

return Paintable