require "prefabs/tunnel"
require "components/paintable"

local tunnel_manager = Class(function(self, inst)
    self.inst = inst
	self.maxPairs = 256
	self.numPairs = 0
	self.orphans = 0
	self.tun_a = nil
	self.tun_b = nil
	self.pairs = {}
	self.col_index = {}
end)

function tunnel_manager:GetPairId()
		return numPairs + 1
end

function tunnel_manager:Increment()
	self.numPairs = self.numPairs + 1
end

function tunnel_manager:Decrement()
	if self.numPairs >= 1 then
		self.numPairs = self.numPairs - 1
	end
end

function tunnel_manager:GetTotalPairs()
	return self.numPairs
end

--I had zero luck just storing RGB values in OnSave and OnLoad for tunnel.lua ... so in desperation I made this
function tunnel_manager:GetKey(inst)
	local x, y, z = inst.Transform:GetWorldPosition()
	local formattedkey = tostring(_G.Truncate(x))..":"..tostring(_G.Truncate(y))..":"..tostring(_G.Truncate(z))
	return formattedkey
end

function tunnel_manager:AddToIndex(inst, tm_data)
	local thekey = self:GetKey(inst)
	--print("Adding.."..thekey)
	
	self.col_index[thekey] = 	{ 
									["r"] 		= tm_data.r, 
									["g"] 		= tm_data.g, 
									["b"] 		= tm_data.b,
								}
end

function tunnel_manager:PrepTunnel(inst)
	inst.components.paintable:Paint(self:GetFromIndex(inst))
end

function tunnel_manager:GetFromIndex(inst)
	local thekey = self:GetKey(inst)
	--print("Getting.."..thekey)
	return self.col_index[thekey]
end

function tunnel_manager:OnSave()
	local data = {}
	data.numPairs = self.numPairs
	data.col_index = {}
	
	for k, v in pairs(self.col_index) do
		data.col_index[k] = self.col_index[k]
	end
	
	return data
end

function tunnel_manager:OnLoad(data)
	if data then
		self.numPairs = data.numPairs
		self.curOrphan = data.curOrphan
		
		if data.col_index then
			for k, v in pairs(data.col_index) do
				self.col_index[k] = data.col_index[k]
				--print("??: "..k)
			end
		end
	end
end

function tunnel_manager:Init()
	
end

function tunnel_manager:MakePairs()
	if self.orphans > 1 then
		local r = math.random(1,255)
		local g = math.random(1,255) 
		local b = math.random(1,255)
		
		local tm_data = { ["r"] = r, ["g"] = g, ["b"] = b }
		
		self:AddToIndex(self.tun_a, tm_data)
		self:AddToIndex(self.tun_b, tm_data)
		
		self.tun_a.components.tunnelteleporter:Target(self.tun_b)
		self.tun_b.components.tunnelteleporter:Target(self.tun_a)
		
		--Paint flag color
		self:PrepTunnel(self.tun_a)
		self:PrepTunnel(self.tun_b)
		
		--Increment numPairs
		self:Increment()
		
		self.orphans = 0
		self.tun_a = nil
		self.tun_b = nil
	end
end

--function tunnel_manager:OnTunnelRemoved(inst)
	--iterate over tunnels and remove links to non-existent ones
--end

function tunnel_manager:InitTunnel(inst)
	if self:GetFromIndex(inst) then
		--print("OLD!")
		self:PrepTunnel(inst)
	else		
		if self.orphans < 1 then
			self.tun_a = inst
		else
			self.tun_b = inst
		end
					
		self.orphans = self.orphans + 1
	end
		
	self:MakePairs()
end

function tunnel_manager:CanMake()
	return self.numPairs <= self.maxPairs 
end

return tunnel_manager