require "prefabutil"

local assets =
{
	Asset("ANIM", 			"anim/tunnel.zip" ),
	Asset("SOUNDPACKAGE", 	"sound/tunnel.fev"),
    Asset("SOUND", 			"sound/tunnel.fsb"),
}

local prefabs =
{
    "prefabs/flag",
}   

local function GetStatus(inst)
	if inst.sg.currentstate.name ~= "idle" then
		return "OPEN"
	end
end

local function OnHit(inst, worker)
	if IsDLCEnabled(REIGN_OF_GIANTS) == "enabled" then 
		if not inst:HasTag("burnt") then
			inst.AnimState:PlayAnimation("hit")
		end
	else
		inst.AnimState:PlayAnimation("hit")
	end
end

local function OnHammered(inst, worker)
	if IsDLCEnabled(REIGN_OF_GIANTS) == "enabled" then 
		if inst:HasTag("fire") and inst.components.burnable then
			inst.components.burnable:Extinguish()
		end
	end
	
	inst.components.lootdropper:DropLoot()
	SpawnPrefab("collapse_big").Transform:SetPosition(inst.Transform:GetWorldPosition())
	inst.SoundEmitter:PlaySound("dontstarve/common/destroy_wood")
	
	inst:RemoveComponent("tunnelteleporter")
	inst:Remove()
end

local function onsave(inst, data)
	if inst:HasTag("burnt") or inst:HasTag("fire") then
        data.burnt = true
    end
end

local function onload(inst, data)
	if data and data.burnt then
        inst.components.burnable.onburnt(inst)
    end
	
	inst:ListenForEvent("onremove", OnRemove(inst))
end

local function onremove(inst)
	inst:RemoveEventCallback("onremove", onremove, inst)
end

local function onbuilt(inst)
	inst:RemoveEventCallback("onbuilt", onbuilt, inst)
	
	--We have to wait because they don't have coordinates yet
	inst:DoTaskInTime(1, function(inst) 
		GetWorld().components.tunnel_manager:InitTunnel(inst)
	end)
end



local function fn(Sim)
	local inst = CreateEntity()
	local trans = inst.entity:AddTransform()
	local anim = inst.entity:AddAnimState()
	
    inst.entity:AddSoundEmitter()
	inst:AddTag("tunnel") 
    
    local minimap = inst.entity:AddMiniMapEntity()
    minimap:SetIcon( "minimap_tunnel.tex" )
   
    anim:SetBank("tunnel")
    anim:SetBuild("tunnel")
	anim:SetRayTestOnBB(true)
	
	inst:AddComponent("paintable")
    inst.components.paintable:AddThing("flag")
	inst.components.paintable:SetParent(inst)
	inst.components.paintable:Paint()
	
	inst:SetStateGraph("SGtunnel")
	
	inst:AddTag("structure")
 
	inst:AddComponent("lootdropper")
	inst:AddComponent("workable")
    inst.components.workable:SetWorkAction(ACTIONS.HAMMER)
    inst.components.workable:SetWorkLeft(4)
	inst.components.workable:SetOnWorkCallback(OnHit)
	inst.components.workable:SetOnFinishCallback(OnHammered)
	
    inst:AddComponent("inspectable")
	inst.components.inspectable.getstatus = GetStatus
	inst.components.inspectable:RecordViews()

	inst:AddComponent("playerprox")
	inst.components.playerprox:SetDist(4,5)
	inst.components.playerprox.onnear = function()
		--if inst.components.teleporter.targetTeleporter ~= nil and not inst.sg:HasStateTag("open") then
			if inst.components.tunnelteleporter.target then
				inst.sg:GoToState("opening")
				_G.DisableWormholeJumpNoise()
			end
		--end
	end
	
	inst.components.playerprox.onfar = function()
		if inst.sg.currentstate.name == "open" then
			inst.sg:GoToState("closing")
			_G.EnableWormholeJumpNoise()
		end
		--inst.SoundEmitter:PlaySound("dontstarve/tunnel/common/close")
	end

	inst:AddComponent("tunnelteleporter")
	--inst.components.teleporter.onActivate = OnActivate
	--inst.components.teleporter.onActivateOther = OnActivateOther

	inst:AddComponent("inventory")

	inst:AddComponent("trader")
	inst.components.trader.onaccept = function(reciever, giver, item)
		-- pass this on to our better half
		reciever.components.inventory:DropItem(item)
		inst.components.teleporter:Activate(item)
	end
	
	if IsDLCEnabled(REIGN_OF_GIANTS) == "enabled" then 
		MakeLargeBurnable(inst, nil, nil, true)
		MakeLargePropagator(inst)

		inst.OnSave = onsave 
		inst.OnLoad = onload
	end
	
	inst:ListenForEvent( "onbuilt", onbuilt(inst))
	inst:ListenForEvent( "onremove", onremove(inst))
	
	--print("LOOKING...", StateGraphWrangler:Find())

    return inst
end

return Prefab( "common/objects/tunnel", fn, assets), 
	   MakePlacer( "common/tunnel_placer", "tunnel", "tunnel", "place" ) 